/*BEGIN CMVC */
//*************************************************************************
//
//  Workfile: %W%
//  Last update: %E% %U%
//  SCCS path, id: %F% %I%
//
//************************************************************************/
/*END CMVC */
/*BEGIN COPYRIGHT */
//*************************************************************************
//
//  Licensed Materials - Property of IBM
//  WA82 SupportPac
//  (c) Copyright IBM Corp. 2000, 2001, 2002
//  US Government Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract
//  with IBM Corp.
//
//************************************************************************/
/*END COPYRIGHT */
/*BEGIN DISCLAIMER */
//*************************************************************************
//  DISCLAIMER
//  ----------
//  This material contains programming source code for your consideration.
//  These examples have not been thoroughly tested under all conditions.
//  IBM, therefore, cannot guarantee or imply reliability, serviceability,
//  or function of these programs.
//  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
//  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
//  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//  FOR A PARTICULAR PURPOSE.
//************************************************************************/
/*END DISCLAIMER */

package com.ibm.workflow.sample.upes;

import java.util.*;
import java.text.*;
import java.sql.*;
import java.io.*;

public class CustomerUPES extends UPESBaseAdapter
{
  private DateFormat        timestamp;
  private Connection        connection;
  private PreparedStatement queryStatement;
  private PreparedStatement getMaxStatement;
  private PreparedStatement insertStatement;
  private PreparedStatement updateStatement;
  private PreparedStatement getMaxLoanStatement;
  private PreparedStatement insertLoanStatement;
  private PreparedStatement updateLoanStatement;

  // main method.
  // Reads the properties file, initializes the
  // CustomerUPES and finally invokes the main loop run( )
  // --------------------------------------------------------------------

  public static void main( String[] args )
  {
    System.out.println( );
    System.out.println( "MQSeries Workflow WebCredit Example (SupportPac WA82) - CustomerUPES" );
    System.out.println( "--------------------------------------------------------------------" );
    System.out.println( "Version 1.4" );
    System.out.println( );
      
    Properties properties = new Properties( );
    try {
      properties.load( new FileInputStream( "CustomerUPES.properties" ) );

      // Create the new CustomerUPES object
      CustomerUPES customerUPES = new CustomerUPES( properties );
          
      // Invoke the main loop of the UPESBaseSAX object
      // This main loop processes the messages and calls the call-back
      // functions processMessage and returnCode in the CustomerUPES class
      customerUPES.run( );

      // Once the main loop is left the only thing left to do is to
      // shutdown the application gracefully
      customerUPES.shutdown( );
    }
    catch( Exception xcpt ) {
      xcpt.printStackTrace( );
      System.exit( 1 );
    }
  }

  // Constructor. Calls the base constructor, then connects to the
  // database and prepares the SQL statements.
  // --------------------------------------------------------------------
  public CustomerUPES( Properties p )
  {
    super( p.getProperty( "MQ.QueueManagerName"   ),
           p.getProperty( "MQ.QueueName"          ),
           p.getProperty( "MQ.Hostname"           ),
           p.getProperty( "MQ.Port"               ),
           p.getProperty( "MQ.Channel"            ) );
    
    System.out.println( "CustomerUPES initializing" );

    this.timestamp = new SimpleDateFormat( "yyyy-MM-dd" );
    String databaseDriver     = p.getProperty( "Database.Driver",
                                               "COM.ibm.db2.jdbc.app.DB2Driver" );
    String databasePrefix     = p.getProperty( "Database.Prefix",
                                               "jdbc:db2" );
    String userID             = p.getProperty( "Database.UserID"       );
    String password           = p.getProperty( "Database.Password"     );
    String databaseName       = p.getProperty( "Database.Name"         );

    try {
      System.out.println( "  Loading Database Driver: " + databaseDriver );
      Class.forName( databaseDriver );
      System.out.println( "  Connecting to the Database '" + databaseName +
                          "' using UserID '" + userID + "'" );
      if ( userID != null )
        connection = DriverManager.getConnection( databasePrefix + ":" + databaseName,
                                                  userID, password );
      else
        connection = DriverManager.getConnection( databasePrefix + ":" + databaseName );

      System.out.println( "  Preparing SQL Statements" );
      queryStatement = connection.prepareStatement( "SELECT COUNT(CUSTOMER_ID) " +
                                                    "FROM CUSTOMER.CUSTOMER_DATA " +
                                                    "WHERE CUSTOMER_ID=?" );
      getMaxStatement = connection.prepareStatement( "SELECT MAX(CUSTOMER_ID)+1 "+
                                                     "FROM CUSTOMER.CUSTOMER_DATA" );
      insertStatement = connection.prepareStatement( "INSERT INTO CUSTOMER.CUSTOMER_DATA " +
                                                     "VALUES (?, ?, ?, ?, ?, ?, ?, ?, " +
                                                     "?, ?, ?, ?, ?, ?, ?, ?)" );
      updateStatement = connection.prepareStatement( "UPDATE CUSTOMER.CUSTOMER_DATA SET " +
                                                     "FIRSTNAME=?, " +
                                                     "MIDDLENAME=?, " +
                                                     "LASTNAME=?, " +
                                                     "ADDRESS_LINE_1=?, " +
                                                     "ADDRESS_LINE_2=?, " +
                                                     "ADDRESS_LINE_3=?, " +
                                                     "CITY=?, " +
                                                     "STATE=?, " +
                                                     "ZIPCODE=?, " +
                                                     "COUNTRY=?, " +
                                                     "EMAIL_ADDRESS=?, " +
                                                     "PHONE_HOME=?, " +
                                                     "PHONE_BUSINESS=?, " +
                                                     "FAX_HOME=?, " +
                                                     "FAX_BUSINESS=? " +
                                                     "WHERE CUSTOMER_ID=?");

      getMaxLoanStatement = connection.prepareStatement( "SELECT MAX(LOAN_ID)+1 FROM CUSTOMER.LOANS" );
      insertLoanStatement = connection.prepareStatement( "INSERT INTO CUSTOMER.LOANS " +
                                                         "VALUES (?, ?, ?, ?, ?, ? )" );
      updateLoanStatement = connection.prepareStatement( "UPDATE CUSTOMER.LOANS SET " +
                                                         "STATUS=? " +
                                                         "WHERE LOAN_ID=?" );

    }
    catch( Exception xcpt ) {
      xcpt.printStackTrace( );
      System.exit( 1 );
    }
    System.out.println( "CustomerUPES initialized" );
    System.out.println( );
  }

  // Shutdown. This method is called by the base class to clean up
  // any resources when a shutdown message is received
  // --------------------------------------------------------------------
  public void shutdown( )
  {
    System.out.println( "CustomerUPES shutdown" );
    try {
      if ( connection != null ) connection.close( );
    }
    catch( Exception xcpt ) { /* Ignore Exceptions */ }
      
    System.out.println( "CustomerUPES shutdown complete" );
    System.out.println( );

    // Make sure that all the resources in the base class are
    // cleaned up as well
    super.shutdown( );
  }
    
  // returnCode Callback function. Has to be implemented because abstract
  // in base class
  // --------------------------------------------------------------------
  public int returnCode( )
  {
    return 0;
  }
  
  // processSyncActivityMessage Callback function. Has to be implemented
  // because abstract in base class. This function processes the
  // message for the Update_Tables activity.
  // --------------------------------------------------------------------
  public String processSyncActivityInvokeMessage( )
  {
    CustomerData data = getDataFromContainer( );
    String xmlOutputData = null;

    try {
      if ( data.activityName.equals( "Update_Tables" ) )
        xmlOutputData = updateTables( data );
      else System.out.println( "Message for wrong activity ('" +
                               data.activityName +
                               "') received. Expected 'Update_Tables'." );
    }
    catch( Exception xcpt ) {
      xcpt.printStackTrace( );
    }

    return xmlOutputData;
  }

  // processAsyncActivityMessage Callback function. Has to be implemented
  // because abstract in base class. This function processes the
  // message for the Update_Loans activity.
  // --------------------------------------------------------------------
  public void processAsyncActivityInvokeMessage( )
  {
    CustomerData data = getDataFromContainer( );

    try {
      if ( data.activityName.equals( "Update_Loans" ) )
        updateLoans( data );
      else System.out.println( "Message for wrong activity ('" +
                               data.activityName +
                               "') received. Expected 'Update_Loans'." );
    }
    catch( Exception xcpt ) {
      xcpt.printStackTrace( );
    }
  }

  // updateTables. Implement the Update_Tables activity.
  // --------------------------------------------------------------------
  private String updateTables( CustomerData data )
    throws SQLException
  {
    System.out.println( "Update_Tables Activity" );
    System.out.println( "  Customer ID: " + data.customerID );

    // Find out if this customer ID exists in the database
    // If it does -> update, else new customer
    queryStatement.setLong(1, data.customerID );
    ResultSet result = queryStatement.executeQuery( );
    result.next( );
    if ( result.getLong( 1 ) == 1 ) {
      System.out.println( "  Update customer... " );

      updateStatement.setString(  1, data.firstName     );
      updateStatement.setString(  2, data.middleName    );
      updateStatement.setString(  3, data.lastName      );
      updateStatement.setString(  4, data.addressLine1  );
      updateStatement.setString(  5, data.addressLine2  );
      updateStatement.setString(  6, data.addressLine3  );
      updateStatement.setString(  7, data.city          );
      updateStatement.setString(  8, data.state         );
      updateStatement.setString(  9, data.zipCode       );
      updateStatement.setString( 10, data.country       );
      updateStatement.setString( 11, data.emailAddress  );
      updateStatement.setString( 12, data.phoneHome     );
      updateStatement.setString( 13, data.phoneBusiness );
      updateStatement.setString( 14, data.faxHome       );
      updateStatement.setString( 15, data.faxBusiness   );
      updateStatement.setLong(   16, data.customerID    );

      updateStatement.execute( );
    }
    else {
      // Insert new customer...
      System.out.println( "  New customer..." );

      // First get the maximum customer ID and increase it by one.
      ResultSet maxResult = getMaxStatement.executeQuery( );
      maxResult.next( );
      data.customerID = maxResult.getLong( 1 );
      if ( data.customerID == 0 ) data.customerID = 1;

      System.out.println( "  New CustomerID: " + data.customerID );
      insertStatement.setLong(    1, data.customerID    );
      insertStatement.setString(  2, data.firstName     );
      insertStatement.setString(  3, data.middleName    );
      insertStatement.setString(  4, data.lastName      );
      insertStatement.setString(  5, data.addressLine1  );
      insertStatement.setString(  6, data.addressLine2  );
      insertStatement.setString(  7, data.addressLine3  );
      insertStatement.setString(  8, data.city          );
      insertStatement.setString(  9, data.state         );
      insertStatement.setString( 10, data.zipCode       );
      insertStatement.setString( 11, data.country       );
      insertStatement.setString( 12, data.emailAddress  );
      insertStatement.setString( 13, data.phoneHome     );
      insertStatement.setString( 14, data.phoneBusiness );
      insertStatement.setString( 15, data.faxHome       );
      insertStatement.setString( 16, data.faxBusiness   );

      insertStatement.execute( );
    }

    // Now get a new Loan ID and insert the loan into the database.
    ResultSet maxLoanResult = getMaxLoanStatement.executeQuery( );
    maxLoanResult.next( );
    data.loanID = maxLoanResult.getLong( 1 );
    if ( data.loanID == 0 ) data.loanID = 1;
    System.out.println( "  New LoanID: " + data.loanID + ", CreditAmount: " + data.creditAmount );

    insertLoanStatement.setLong(   1, data.loanID       );
    insertLoanStatement.setLong(   2, data.customerID   );
    insertLoanStatement.setString( 3, timestamp.format( new java.util.Date( ) ) );
    insertLoanStatement.setDouble( 4, data.creditAmount );
    insertLoanStatement.setString( 5, data.creditReason );
    insertLoanStatement.setString( 6, "In progress"     );

    insertLoanStatement.execute( );

    StringBuffer msgText = new StringBuffer( );
    msgText.append( "<WebCredit_Data>" + eol );
    msgText.append( "  <CustomerID>" + data.customerID + "</CustomerID>" + eol );
    msgText.append( "  <LoanID>" + data.loanID + "</LoanID>" + eol );
    msgText.append( "</WebCredit_Data>" + eol );
    return msgText.toString( );
  }

  // updateLoans. Implements the Update_Loans activity.
  // --------------------------------------------------------------------
  private void updateLoans( CustomerData data )
    throws SQLException
  {
    System.out.println( "Updating Loans" );
    if ( data.approved == 1 ) {
      updateLoanStatement.setString( 1, "Approved" );
      System.out.println( "  Loan approved. LoanID: " + data.loanID );
    }
    else {
      updateLoanStatement.setString( 1, "Rejected" );
      System.out.println( "  Loan rejected. LoanID: " + data.loanID );
    }

    updateLoanStatement.setLong( 2, data.loanID );
    updateLoanStatement.execute( );
  }
    
  // --------------------------------------------------------------------
  private CustomerData getDataFromContainer( )
  {
    Map          inContainer = getInContainer( );
    CustomerData data        = new CustomerData( );

    data.activityName  = getString( inContainer, "_ACTIVITY"      );
    data.customerID    = getLong(   inContainer, "CustomerID"     );
    data.loanID        = getLong(   inContainer, "LoanID"         );
    data.creditAmount  = getDouble( inContainer, "CreditAmount"   );
    data.approved      = getLong(   inContainer, "Approved"       );
    data.creditReason  = getString( inContainer, "CreditReason"   );
    data.firstName     = getString( inContainer, "FirstName"      );
    data.middleName    = getString( inContainer, "MiddleName"     );
    data.lastName      = getString( inContainer, "LastName"       );
    data.addressLine1  = getString( inContainer, "Address_Line1"  );
    data.addressLine2  = getString( inContainer, "Address_Line2"  );
    data.addressLine3  = getString( inContainer, "Address_Line3"  );
    data.city          = getString( inContainer, "City"           );
    data.state         = getString( inContainer, "State"          );
    data.zipCode       = getString( inContainer, "Zipcode"        );
    data.country       = getString( inContainer, "Country"        );
    data.emailAddress  = getString( inContainer, "Email_Address"  );
    data.phoneHome     = getString( inContainer, "Phone_Home"     );
    data.phoneBusiness = getString( inContainer, "Phone_Business" );
    data.faxHome       = getString( inContainer, "Fax_Home"       );
    data.faxBusiness   = getString( inContainer, "Fax_Business"   );

    return data;
  }

  // --------------------------------------------------------------------
  private class CustomerData
  {
    public String activityName    = "";
    public long   approved        = 0;
    public long   customerID      = 0;
    public long   loanID          = 0;
    public String firstName       = "";
    public String middleName      = "";
    public String lastName        = "";
    public String addressLine1    = "";
    public String addressLine2    = "";
    public String addressLine3    = "";
    public String city            = "";
    public String state           = "";
    public String zipCode         = "";
    public String country         = "";
    public String emailAddress    = "";
    public String phoneHome       = "";
    public String phoneBusiness   = "";
    public String faxHome         = "";
    public String faxBusiness     = "";

    public String creditReason    = "";
    public double creditAmount    = 0.0;
  }
}
